#!/usr/bin/perl
#
# SysWatch v1.4.3 - CGI Script to gather and report various system information.
# Copyright (C) 2000  Chris Martino <smirks@smirks.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

# All configuration is now done to the syswatch.cf file.

$config = "./syswatch.cf";
require "$config";

##### Do NOT edit below here unless you know what you're doing!!  #####
# ------------------------------------------------------------------- #

my $temp_command;
my @devinfo;
my @df_info;
my $info;
my @show_list;
my $dev_count= 0;
my $devline;
my $bad_line= "";

my $sysos = `uname -sr`;
my $uptime = `uptime`;
my $tmptop = '/tmp/tmptop';
my $tmpw = '/tmp/tmpw';
my $tmptail = '/tmp/tmptail';

if( $sysos =~ /linux/i ) {
	$sysname = `hostname -f`;
} else {
	$sysname = `hostname`;
};

if ( $sysos =~ /freebsd/i ) {
        $memory = `expr \`cat /var/run/dmesg.boot | grep "real memory" | cut -f5 -d" "\` \/ 1048576`;
        $swap = `$toploc | grep -i swap | cut -f2 -d" " | cut -f1 -d"M"`;
	chomp($memory);
	chomp($swap);
} elsif ( $sysos =~ /netbsd/i ) {
        $memory = `expr \`sysctl -a | grep hw.physmem | cut -c13-\` \/ 1048576`;
        $swap = `expr \`swapctl -sk | sed 'y/k/ /' | awk '{ print \$2 }'\` \/ 1024`;
        chomp($swap);
        chomp($memory);
} elsif ( $sysos =~ /linux/i ) {
	$memory = `free -m |grep -i mem | awk '{print \$2}'`;
	$swap = `free -m |grep -i swap | awk '{print \$2}'`;
        chomp($swap);
        chomp($memory);
} elsif ( $sysos =~ /sunos/i ) {
	$memgather = `$toploc | grep Mem`;
	if ( $memgather =~ /^Memory\: (\d+)M real.*(\d+)M swap in use, (\d+)M swap/ ) {
		$memory = $1;
		$swap = $2 + $3;
	} else {
		$memory = $memory;
		$swap = $swap;
	}
} else {
	$memory = $memory;
	$swap = $swap;
};

$df_command= "df -k";
@devinfo = `$df_command`;

foreach $devline ( @devinfo ) {
  
  chomp( $devline );
  
  if( $bad_line ne "" ) {
    $df_info[0]= $bad_line;
    ($df_info[1], $df_info[2], $df_info[3], $df_info[4], $df_info[5] )
      = split( ' ', $devline);   
    $bad_line = "";   
  } else {
    @df_info = split( ' ', $devline);
  }
    
  if( $df_info[1] ne "" ) {
    $bad_line = "";
  } else {
    # Okay we proably have a long line for the devices so df has wraped the line ARG.
    $bad_line= $df_info[0]; # save this for next pass
  };
    
  if ((not (( $df_info[0] =~ /^Filesystem/ ) ||
      ( in( $df_info[0], @exclude_devs ) ))) &&
      ( $bad_line eq "")) {
  
    if( not(( $exclude_nfs eq "TRUE" ) && ( $df_info[0] =~ /^.*:.*$/ )) ) {

      $show_list[ $dev_count++]=[ @df_info ];
    };
  }
}

chomp($uptime);
chomp($sysos);
chomp($sysname);

if ( $sysos =~ /hpux/i ) {
	open(OUTF,">>$tmptop");
        	print OUTF `ps aux |grep -A50 PID |grep -v grep`;
	close(OUTF);
} elsif ( $sysos =~ /linux/i ) {
	open(OUTF,">>$tmptop");
		print OUTF `$toploc b n 1 |grep -A50 PID`;
	close(OUTF);
} else {
	print OUTF `$toploc -o cpu -d1 50`;
	close(OUTF);
};

open(OUTF,">>$tmpw");
	print OUTF `w`;
close(OUTF);

open(OUTF,">>$tmptail");
	print OUTF " ";
	print OUTF `tail -n $taillines $logs`;
close(OUTF);
print "Content-type: text/html\n\n";

print <<EndHTML;
<html>
<head>
<title>SysWatch -> $sysname</title>
</head>
<body bgcolor=#000000 text="#ffffff">
<center>
<table cellpadding="0" cellspacing="0" width="60%" border="0">
</table>
<table bgcolor=#000000 width="60%" border=0 cellpadding=2 cellspacing=0>
<tr>
<td width="60%" height="23" bgcolor=#231d53 colspan=1>
<font color=#ffffff  face="arial, helvetica" size="2">
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;System Information
</font>
</td>   
</tr>
<tr><td><font face="courier" size="2"><pre>
SysInfo    : $sysname
&nbsp;   $uptime

OS	: $sysos
RAM	: $memory MB (approx.)
Swap	: $swap MB (approx.)
</pre></font></td></tr>
</table>
EndHTML
;
if ( $showdrivemap == 1 ) {
	print "<table width=\"60%\" bgcolor=#000000 border=0 cellpadding=2 cellspacing=0>\n";
	print "<tr>\n";
	print "<td height=\"23\" bgcolor=#231d53 colspan=6>\n";
	print "<font color=#ffffff  face=\"arial, helvetica\" size=\"2\">\n";
	print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Drive Information\n";
	print "</font>\n";
	print "</td>\n";   
	print "</tr>\n";
	print "<tr>\n";
	print "<td align=center><font face=\"arial, helvetica\" size=\"2\"><b>Filesystem</b></font></td>\n";
	print "<td align=center><font face=\"arial, helvetica\" size=\"2\"><b>Mount</b></font></td>\n";
	print "<td align=center><font face=\"arial, helvetica\" size=\"2\"><b>Use</b></font></td>\n";
	print "<td align=center><font face=\"arial, helvetica\" size=\"2\"><b>Used</b></font></td>\n";
	print "<td align=center><font face=\"arial, helvetica\" size=\"2\"><b>Avail</b></font></td>\n";
	print "<td align=center><font face=\"arial, helvetica\" size=\"2\"><b>Size</b></font></td>\n";
	if ( $dev_count > 0 ) {
	  foreach $item ( @show_list ) {
	    $disk_full= 0; # Used to graph the amount of disk space full
	    print "<tr>\n";
	    print "<td align=left><font face=\"courier\" size=\"2\">@{$item}[0]</font></td>\n";
	    print "<td align=left><font face=\"courier\" size=\"2\">@{$item}[5]</font></td>\n";
	    print "<td align=left><font face=\"courier\" size=\"2\">\n";
	    $disk_full= @{$item}[4];
	    chop( $disk_full);
	    if( $disk_full < 90 ) {
	      if( $disk_full == 0 ) {
	        print "@{$item}[4]</font></td>\n";
	      } else {
	      print "<img src=\"$image\" height=10 width=$disk_full> @{$item}[4]</font></td>\n";
	      };
	    } else {
	        print "<img src=\"$full_image\" height=10 width=$disk_full> @{$item}[4]</font></td>\n";
	    };
	    print "<td align=right><font face=\"courier\" size=\"2\">@{$item}[2]K</font></td>\n";
	    print "<td align=right><font face=\"courier\" size=\"2\">@{$item}[3]K</font></td>\n";
	    print "<td align=right><font face=\"courier\" size=\"2\">@{$item}[1]K</font></td>\n";
	    print "</tr>\n";
	  }
	} else {
	   print "<tr>\n";
	   print "<td align=center colspan=6><font face=\"arial, helvetica\" size=\"2\">No Disk info avaible</font></td>\n";
	   print "</tr>\n";
	 }
	print "</table></p>";
}
if ( $showwhat == 1 ) {
	print "<p>\n";
	print "<table bgcolor=#000000 width=\"60%\" border=0 cellpadding=2 cellspacing=0>\n";
	print "<tr>\n";
	print "<td width=\"60%\" height=\"23\" bgcolor=#231d53 colspan=1>\n";
	print "<font color=#ffffff  face=\"arial, helvetica\" size=\"2\">\n";
	print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Who, What, When\n";
	print "</font>\n";
	print "</td>\n";
	print "</tr>\n";
	print "<tr><td><font face=\"courier\" size=\"2\"><pre>\n";
	open(INF, "$tmpw");
		@w = <INF>;
	close(INF);
	print "\n@w";
	print "</pre></font></td></tr></table>";
}
if ( $showhogs == 1 ) {
	print "<table bgcolor=#000000 width=\"60%\" border=0 cellpadding=2 cellspacing=0>\n";
	print "<tr>\n";
	print "<td width=\"60%\" height=\"23\" bgcolor=#231d53 colspan=1>\n";
	print "<font color=#ffffff  face=\"arial, helvetica\" size=\"2\">\n";
	print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Resource Information\n";
	print "</font>\n";
	print "</td>\n";
	print "</tr>\n";
	print "<tr><td><font face=\"courier\" size=\"2\"><pre>";
	open(INF, "$tmptop");
		@top = <INF>;
	close(INF);
	print "@top";
	print "</pre></font></td></tr></table>";
}
if ( $showtail == 1 ) {
	print "\n";
	print "<table bgcolor=#000000 width=\"60%\" border=0 cellpadding=2 cellspacing=0>\n";
	print "<tr>\n";
	print "<td width=\"60%\" height=\"23\" bgcolor=#231d53 colspan=1>\n";
	print "<font color=#ffffff  face=\"arial, helvetica\" size=\"2\">\n";
	print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;System Messages\n";
	print "</font>\n";
	print "</td>\n";
	print "</tr>\n";
	print "<tr><td><font face=\"arial, helvetica\" size=\"2\"><pre>\n";
	open(INF, "$tmptail");
		@tail = <INF>;
	close(INF);
	print "\n@tail";
	print "</pre></font></td></tr></table>";
}
print "<hr><font face=\"arial\" size=1>Output produced by <a href=\"http://www.smirks.org/syswatch\">SysWatch 1.4.3</a></font>\n";
print "</body></html>\n";
`rm $tmptop`;
`rm $tmpw`;
`rm $tmptail`;

sub in { 

    $this= shift;
    @here=@_; 
    $found_it=0;

    for( $num=0; (($num < scalar( @here)) && ( ! $found_it )); $num++) {
        if( $here[ $num] eq $this ) {
            $found_it=1;
        };
    };
    return $found_it;
};

